from collections import namedtuple
from datetime import date, datetime

from django.utils.functional import cached_property, lazy
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from ..utils import currency, paragraph


class PaymentStatusMixin:
    @property
    def receivable(self):
        return self.price - self.discount

    @property
    def balance(self):
        return self.paid - self.receivable

    @property
    def color(self):
        if self.overdue:
            return settings.LEPRIKON_COLOR_OVERDUE
        if self.amount_due:
            return settings.LEPRIKON_COLOR_NOTPAID
        if self.overpaid:
            return settings.LEPRIKON_COLOR_OVERPAID
        return settings.LEPRIKON_COLOR_PAID

    @property
    def amount_due_color(self):
        if self.overdue:
            return settings.LEPRIKON_COLOR_OVERDUE
        if self.amount_due:
            return settings.LEPRIKON_COLOR_NOTPAID
        return settings.LEPRIKON_COLOR_PAID

    @property
    def overpaid_color(self):
        if self.overpaid:
            return settings.LEPRIKON_COLOR_OVERPAID
        return settings.LEPRIKON_COLOR_PAID

    @property
    def title(self):
        titles = []
        if self.amount_due and (self.amount_due != self.overdue):
            titles.append(_('amount due {}').format(currency(self.amount_due)))
        if self.overdue:
            titles.append(_('overdue {}').format(currency(self.overdue)))
        if self.overpaid:
            titles.append(_('{} overpaid').format(currency(self.overpaid)))
        if self.balance == 0:
            if self.receivable:
                titles.append(_('paid'))
            else:
                titles.append(_('payment not requested'))
        elif self.balance < 0 and not self.amount_due:  # self.due_from > self.current_date:
            titles.append(_('payment not requested yet'))
        return ', '.join(map(str, titles))

    def __xrepr__(self):
        return (
            '{type_name}(price={price}, discount={discount}, paid={paid}, balance={balance}, '
            'amount_due={amount_due}, overdue={overdue}, overpaid={overpaid})'.format(
                type_name=type(self).__name__,
                price=self.price,
                discount=self.discount,
                paid=self.paid,
                balance=self.balance,
                amount_due=self.amount_due,
                overdue=self.overdue,
                overpaid=self.overpaid,
            )
        )

    def __add__(self, other):
        return PaymentStatusSum(
            price=self.price + (other and other.price),
            discount=self.discount + (other and other.discount),
            paid=self.paid + (other and other.paid),
            amount_due=self.amount_due + (other and other.amount_due),
            overdue=self.overdue + (other and other.overdue),
            overpaid=self.overpaid + (other and other.overpaid),
        )

    __radd__ = __add__


class PaymentStatus(
    PaymentStatusMixin,
    namedtuple(
        '_PaymentsStatus',
        ('price', 'discount', 'explanation', 'paid', 'current_date', 'due_from', 'due_date'),
    ),
):
    @property
    def amount_due(self):
        return max(self.receivable - self.paid, 0) if self.due_from and self.due_from <= self.current_date else 0

    @property
    def overdue(self):
        return self.amount_due if self.due_date and self.due_date < self.current_date else 0

    @property
    def overpaid(self):
        return max(self.balance, 0)

    def __str__(self):
        return format_html(
            '<strong title="{title}" style="color: {color}">{balance}</strong>',
            color=self.color,
            balance=currency(self.balance),
            title=self.title,
        )


class PaymentStatusSum(
    PaymentStatusMixin,
    namedtuple(
        '_PaymentsStatusSum',
        ('price', 'discount', 'paid', 'amount_due', 'overdue', 'overpaid'),
    ),
):
    pass


class BankAccount:
    def __init__(self, iban):
        self.iban = iban

    @cached_property
    def country_code(self):
        return self.iban[:2]

    @cached_property
    def bank_code(self):
        return self.iban[4:8]

    @cached_property
    def account_prefix(self):
        return self.iban[8:14].lstrip('0')

    @cached_property
    def account_number(self):
        return self.iban[14:].lstrip('0')

    def __str__(self):
        return '%s%s%s/%s' % (
            self.account_prefix,
            self.account_prefix and '-',
            self.account_number,
            self.bank_code,
        )


def generate_variable_symbol(registration):
    # get base variable symol from the configured expression
    variable_symbol = eval(settings.LEPRIKON_VARIABLE_SYMBOL_EXPRESSION, {'reg': registration})

    # add check digit
    odd_sum = 0
    even_sum = 0
    for i, char in enumerate(str(variable_symbol)):
        if i % 2:
            even_sum += int(char)
        else:
            odd_sum += int(char)
    check_digit = (odd_sum * 3 + even_sum) % 10
    return variable_symbol * 10 + check_digit


def help_text_with_html_default(help_text, html_default):
    keep_empty = paragraph(_('Keep empty to use default value:'))
    return mark_safe('{}{}{}'.format(
        paragraph(help_text),
        keep_empty,
        html_default,
    ) if help_text else '{}{}'.format(
        keep_empty,
        html_default,
    ))


lazy_help_text_with_html_default = lazy(help_text_with_html_default, str)


def help_text_with_default(help_text, default):
    keep_empty_default = _('Keep empty to use default value: {}').format(default)
    return paragraph(
        '{}\n\n{}'.format(help_text, keep_empty_default) if help_text else keep_empty_default
    )


lazy_help_text_with_default = lazy(help_text_with_default, str)


def change_year(d, year_delta):
    if isinstance(d, date):
        try:
            return date(d.year + year_delta, d.month, d.day)
        except ValueError:
            # handle leap-year
            return date(d.year + year_delta, d.month, d.day - 1)
    else:
        try:
            return datetime(d.year + year_delta, d.month, d.day)
        except ValueError:
            # handle leap-year
            return datetime(d.year + year_delta, d.month, d.day - 1, d.hour, d.minute, d.second)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-02-09 09:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0042_chat_group_types'),
    ]

    operations = [
        migrations.CreateModel(
            name='RegistrationLink',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(editable=False, max_length=64)),
                ('name', models.CharField(max_length=50, verbose_name='name')),
                ('reg_from', models.DateTimeField(verbose_name='registration active from')),
                ('reg_to', models.DateTimeField(verbose_name='registration active to')),
                ('school_year', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, related_name='registration_links', to='leprikon.SchoolYear', verbose_name='school year')),
                ('subject_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='registration_links', to='leprikon.SubjectType', verbose_name='subject type')),
                ('subjects', models.ManyToManyField(related_name='registration_links', to='leprikon.Subject', verbose_name='subjects')),
            ],
            options={
                'verbose_name': 'registration link',
                'verbose_name_plural': 'registration links',
                'ordering': ('reg_from', 'reg_to'),
            },
        ),
        migrations.AddField(
            model_name='subjectregistration',
            name='registration_link',
            field=models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='registrations', to='leprikon.RegistrationLink', verbose_name='registration link'),
        ),
    ]



from string import ascii_letters, digits

from django.contrib import admin
from django.db import transaction
from django.db.models import Count
from django.db.utils import IntegrityError
from django.http import HttpResponseRedirect
from django.urls import reverse
from django.utils.crypto import get_random_string
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from ..forms.registrationlink import RegistrationLinkAdminForm
from ..models.courses import CourseRegistration
from ..models.events import EventRegistration
from ..models.orderables import OrderableRegistration
from ..models.registrationlink import RegistrationLink
from ..models.subjects import SubjectType
from .export import AdminExportMixin
from .filters import SchoolYearListFilter, SubjectTypeListFilter


@admin.register(RegistrationLink)
class RegistrationLinkAdmin(AdminExportMixin, admin.ModelAdmin):
    _registration_models = {
        SubjectType.COURSE: CourseRegistration,
        SubjectType.EVENT: EventRegistration,
        SubjectType.ORDERABLE: OrderableRegistration,
    }
    list_display = (
        'id', 'name', 'get_link', 'subject_type', 'reg_from', 'reg_to', 'get_registrations_link',
    )
    list_filter = (
        ('school_year', SchoolYearListFilter),
        'subject_type__subject_type',
        ('subject_type', SubjectTypeListFilter),
    )
    filter_horizontal = ('subjects',)

    def changeform_view(self, request, object_id=None, form_url='', extra_context=None):
        if not object_id and request.method == 'POST' and len(request.POST) == 3:
            return HttpResponseRedirect('{}?subject_type={}'.format(
                request.path,
                request.POST.get('subject_type', ''),
            ))
        else:
            return super().changeform_view(request, object_id, form_url, extra_context)

    def get_form(self, request, obj, **kwargs):
        # set school year
        if obj:
            request.school_year = obj.school_year

        # get subject type
        try:
            # first try request.POST (user may want to change subject type)
            request.subject_type = SubjectType.objects.get(id=int(request.POST.get('subject_type')))
        except (SubjectType.DoesNotExist, TypeError, ValueError):
            if obj:
                # use subject type from object
                request.subject_type = obj.subject_type
            else:
                # try to get subject type from request.GET
                try:
                    request.subject_type = SubjectType.objects.get(
                        id=int(request.GET.get('subject_type')),
                    )
                except (SubjectType.DoesNotExist, TypeError, ValueError):
                    request.subject_type = None

        if request.subject_type:
            kwargs['form'] = type(
                RegistrationLinkAdminForm.__name__,
                (RegistrationLinkAdminForm, ),
                {
                    'school_year': request.school_year,
                    'subject_type': request.subject_type,
                },
            )
        else:
            kwargs['fields'] = ['subject_type']

        return super().get_form(request, obj, **kwargs)

    def save_form(self, request, form, change):
        obj = super().save_form(request, form, change)
        obj.school_year = request.school_year
        return obj

    def save_model(self, request, obj, form, change):
        if change:
            obj.save()
        else:
            obj.school_year = request.school_year
            while not obj.id:
                try:
                    with transaction.atomic():
                        obj.slug = get_random_string(64, ascii_letters + digits)
                        obj.save()
                except IntegrityError:
                    pass

    def get_link(self, obj):
        return '<a href="{url}" title="{title}" target="_blank">{url}</a>'.format(
            url=obj.link,
            title=_('registration link'),
        )
    get_link.short_description = _('registration link')
    get_link.allow_tags = True

    def get_queryset(self, request):
        return (
            super()
            .get_queryset(request).select_related('subject_type')
            .annotate(registrations_count=Count('registrations'))
        )

    def get_registrations_link(self, obj):
        registration_model = self._registration_models[obj.subject_type.subject_type]
        return format_html(
            '<a href="{url}">{count}</a>',
            url=reverse('admin:{}_{}_changelist'.format(
                registration_model._meta.app_label,
                registration_model._meta.model_name,
            )) + '?registration_link__id__exact={}'.format(obj.id),
            count=obj.registrations_count,
        )
    get_registrations_link.short_description = _('registrations')
    get_registrations_link.allow_tags = True



# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-29 22:16
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0040_due_dates'),
    ]

    operations = [
        migrations.CreateModel(
            name='BillingInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('street', models.CharField(blank=True, max_length=150, null=True, verbose_name='street')),
                ('city', models.CharField(blank=True, max_length=150, null=True, verbose_name='city')),
                ('postal_code', leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code')),
                ('company_num', models.CharField(blank=True, max_length=8, null=True, verbose_name='company number')),
                ('vat_number', models.CharField(blank=True, max_length=10, null=True, verbose_name='VAT number')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='leprikon_billing_info', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'verbose_name': 'billing information',
                'verbose_name_plural': 'billing information',
                'ordering': ('name',),
            },
        ),
        migrations.CreateModel(
            name='SubjectRegistrationBillingInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('street', models.CharField(blank=True, max_length=150, null=True, verbose_name='street')),
                ('city', models.CharField(blank=True, max_length=150, null=True, verbose_name='city')),
                ('postal_code', leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code')),
                ('company_num', models.CharField(blank=True, max_length=8, null=True, verbose_name='company number')),
                ('vat_number', models.CharField(blank=True, max_length=10, null=True, verbose_name='VAT number')),
                ('registration', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='billing_info', to='leprikon.SubjectRegistration', verbose_name='registration')),
            ],
            options={
                'verbose_name': 'billing information',
                'verbose_name_plural': 'billing information',
            },
        ),
    ]



from django import forms
from django.contrib import admin
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from ..forms.journals import JournalEntryAdminForm, JournalLeaderEntryAdminForm
from ..models.journals import JournalEntry, JournalLeaderEntry
from .export import AdminExportMixin
from .filters import LeaderListFilter, SchoolYearListFilter, SubjectListFilter


@admin.register(JournalLeaderEntry)
class JournalLeaderEntryAdmin(AdminExportMixin, admin.ModelAdmin):
    form = JournalLeaderEntryAdminForm
    list_display = ('timesheet', 'date', 'start', 'end', 'duration', 'subject')
    list_filter = (('timesheet__leader', LeaderListFilter),)
    ordering = ('-journal_entry__date', '-start')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj and obj.timesheet.submitted:
            return False
        return super(JournalLeaderEntryAdmin, self).has_delete_permission(request, obj)

    def get_readonly_fields(self, request, obj=None):
        if obj and obj.timesheet.submitted:
            return ('start', 'end')
        return self.readonly_fields


class JournalLeaderEntryInlineAdmin(admin.TabularInline):
    class form(forms.ModelForm):
        class Meta:
            model = JournalLeaderEntry
            fields = []
    model = JournalLeaderEntry
    ordering = ('journal_entry__date', 'start')
    readonly_fields = ('date', 'start', 'end', 'edit_link')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj:
            # obj may be Timesheet or JournalEntry
            # this inline is used in both JournalEntryAdmin and TimesheetAdmin
            try:
                entries = obj.leader_entries
            except AttributeError:
                entries = obj.journal_entries
            if entries.filter(timesheet__submitted=True).exists():
                return False
        return super(JournalLeaderEntryInlineAdmin, self).has_delete_permission(request, obj)

    def edit_link(self, obj):
        return '<a href="{url}" title="{title}" target="_blank">{edit}</a>'.format(
            url=reverse('admin:leprikon_journalleaderentry_change', args=[obj.id]),
            title=_('update entry'),
            edit=_('edit'),
        )
    edit_link.short_description = ''
    edit_link.allow_tags = True


@admin.register(JournalEntry)
class JournalEntryAdmin(AdminExportMixin, admin.ModelAdmin):
    form = JournalEntryAdminForm
    date_hierarchy = 'date'
    list_display = ('id', 'subject_name', 'date', 'start', 'end', 'duration', 'agenda_html')
    list_filter = (
        ('subject__school_year', SchoolYearListFilter),
        ('subject', SubjectListFilter),
    )
    filter_horizontal = ('participants',)
    inlines = (JournalLeaderEntryInlineAdmin,)
    ordering = ('-date', '-start')
    readonly_fields = ('subject_name', 'date',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if obj:
            if obj.affects_submitted_timesheets:
                return False
            else:
                return super().has_delete_permission(request, obj)
        return False

    def get_actions(self, request):
        actions = super().get_actions(request)
        if 'delete_selected' in actions:
            def delete_selected(model_admin, request, queryset):
                objs = [
                    journal_entry
                    for journal_entry in queryset.all()
                    if not journal_entry.affects_submitted_timesheets
                ]
                return admin.actions.delete_selected(model_admin, request, objs)
            actions['delete_selected'] = (
                delete_selected,
                *actions['delete_selected'][1:]
            )
        return actions

    def subject_name(self, obj):
        return obj.subject.name
    subject_name.short_description = _('subject')
    subject_name.admin_order_field = 'subject__name'

    def agenda_html(self, obj):
        return obj.agenda
    agenda_html.short_description = _('agenda')
    agenda_html.admin_order_field = 'agenda'
    agenda_html.allow_tags = True



from collections import namedtuple

from django import forms
from django.utils.translation import ugettext_lazy as _


class TextField(forms.CharField):
    widget = forms.Textarea


class AgreementBooleanField(forms.ChoiceField):
    def __init__(self, allow_disagree=False, **kwargs):
        empty_label = kwargs.pop('empty_label', '-')
        agree_label = kwargs.pop('agree_label', _('agree'))
        disagree_label = kwargs.pop('disagree_label', _('disagree'))
        kwargs['choices'] = [
            ('', empty_label),
            (True, agree_label),
            (False, disagree_label),
        ] if allow_disagree else [
            ('', empty_label),
            (True, agree_label),
        ]
        super(AgreementBooleanField, self).__init__(**kwargs)

    def to_python(self, value):
        if value == 'True':
            value = True
        elif value == 'False':
            value = False
        else:
            value = None
        return value


ReadonlyField = namedtuple('ReadonlyField', ('label', 'value'))



from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext as _

from ..models.schoolyear import SchoolYearBlockPlugin
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonSchoolYearBlockPlugin(LeprikonPluginBase):
    name = _('School year block')
    model = SchoolYearBlockPlugin
    allow_children = True
    filter_horizontal = ('school_years',)
    render_template = 'leprikon/cms/school_year_block.html'



from django.db import models
from django.utils.translation import ugettext_lazy as _


class AccountClosureManager(models.Manager):

    def max_closure_date(self):
        return self.aggregate(closure_date=models.Max('closure_date'))['closure_date']


class AccountClosure(models.Model):
    closure_date = models.DateField(_('closure date'), db_index=True)

    objects = AccountClosureManager()

    class Meta:
        app_label = 'leprikon'
        ordering = ('-closure_date',)
        verbose_name = _('account closure')
        verbose_name_plural = _('account closures')

    def __str__(self):
        return str(self.closure_date)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-11-30 13:32
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0036_participants_instructed'),
    ]

    operations = [
        migrations.AddField(
            model_name='message',
            name='sender',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='sender'),
        ),
    ]



import time
import traceback

from django.core.management.base import BaseCommand
from django.utils.translation import activate
from requests import get
from requests.exceptions import ConnectionError

from leprikon.conf import settings
from leprikon.rocketchat import RocketChat


class Command(BaseCommand):
    help = 'Sync all users, groups and settings with RocketChat'

    def handle(self, *args, **options):
        rocket_chat_ready = False
        while not rocket_chat_ready:
            try:
                rocket_chat_ready = get(settings.ROCKETCHAT_API_URL).ok
            except ConnectionError as e:
                self.stdout.write(self.style.ERROR(repr(e)))
                self.stdout.write(
                    f'Waiting for RocketChat at {settings.ROCKETCHAT_API_URL}'
                )
                time.sleep(5)
        activate(settings.LANGUAGE_CODE)
        rc = RocketChat()
        self.call(rc.configure)
        self.call(rc.sync_roles)
        self.call(rc.sync_users)
        self.call(rc.sync_subjects)

    def call(self, func):
        self.stdout.write(f'calling RocketChat.{func.__name__}')
        try:
            result = func()
        except Exception:
            self.stdout.write(self.style.ERROR('failed'))
            traceback.print_exc()
        else:
            self.stdout.write(self.style.SUCCESS(f'OK ({repr(result)})'))



from django import forms
from django.contrib.admin.widgets import FilteredSelectMultiple
from django.utils.translation import ugettext_lazy as _

from ..models.courses import CourseDiscount, CourseRegistrationPeriod
from ..models.schoolyear import SchoolYearPeriod
from .subjects import RegistrationAdminForm


class CourseRegistrationAdminForm(RegistrationAdminForm):
    periods = forms.ModelMultipleChoiceField(
        queryset=SchoolYearPeriod.objects.all(),
        widget=FilteredSelectMultiple(
            verbose_name=_('Periods'),
            is_stacked=False,
        ),
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.available_periods = self.subject.course.school_year_division.periods.all()
        self.fields['periods'].widget.choices.queryset = self.available_periods
        if self.instance:
            self.fields['periods'].initial = self.available_periods.filter(
                course_registration_periods__registration=self.instance,
            )

    def _save_m2m(self):
        super()._save_m2m()
        for period in self.cleaned_data['periods']:
            CourseRegistrationPeriod.objects.get_or_create(
                registration=self.instance,
                period=period,
            )
        self.instance.course_registration_periods.exclude(
            period__in=self.cleaned_data['periods']
        ).delete()
        return self.instance


class CourseDiscountAdminForm(forms.ModelForm):

    class Meta:
        model = CourseDiscount
        fields = ['registration', 'registration_period', 'accounted', 'amount', 'explanation']

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if 'registration_period' in self.fields:
            try:
                if self.data:
                    registration_id = int(self.data['registration'])
                elif self.initial:
                    registration_id = int(self.initial['registration'])
                else:
                    registration_id = self.instance.registration_id
            except Exception:
                registration_id = None
            self.fields[
                'registration_period'
            ].widget.choices.queryset = self.fields[
                'registration_period'
            ].widget.choices.queryset.filter(
                registration_id=registration_id,
            )



from django.contrib.auth import (
    get_user_model, login as auth_login, views as auth_views,
)
from django.urls import reverse_lazy
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from ..forms.user import (
    PasswordResetForm, SetPasswordForm, UserAgreementForm, UserCreateForm,
    UserEmailForm, UserLoginForm, UserPasswordForm, UserUpdateForm,
)
from ..models.roles import Parent
from ..models.useragreement import UserAgreement
from .generic import CreateView, FormView, UpdateView


class UserCreateView(CreateView):
    model = get_user_model()
    form_class = UserCreateForm
    title = _('Create account')
    form_item_template_name = 'leprikon/user_create_form_item.html'

    def get_message(self):
        return _('User account {} has been created.').format(self.object)

    def form_valid(self, form):
        response = super(UserCreateView, self).form_valid(form)

        # first user is superuser
        if self.model.objects.count() == 1:
            self.object.is_staff = True
            self.object.is_superuser = True
            self.object.save()

        # save agreement
        UserAgreement.objects.create(
            user=self.object,
            granted=now(),
        )

        # create default parent
        parent = Parent()
        parent.first_name = self.object.first_name
        parent.last_name = self.object.last_name
        parent.email = self.object.email
        parent.user = self.object
        parent.save()

        # login
        self.object.backend = 'django.contrib.auth.backends.ModelBackend'
        auth_login(self.request, self.object)
        return response


class UserAgreementView(FormView):
    form_class = UserAgreementForm
    title = _('Agreement with the terms of use')
    form_item_template_name = 'leprikon/user_create_form_item.html'

    def get_message(self):
        return _('The agreement has been saved.')

    def form_valid(self, form):
        try:
            self.request.user.agreement.granted = now()
            self.request.user.agreement.save()
        except UserAgreement.DoesNotExist:
            UserAgreement.objects.create(
                user=self.request.user,
                granted=now(),
            )
        return super(UserAgreementView, self).form_valid(form)


class UserUpdateView(UpdateView):
    model = get_user_model()
    form_class = UserUpdateForm
    title = _('Change user')

    def get_object(self):
        return self.request.user

    def get_message(self):
        return _('User account {} has been updated.').format(self.object)


class UserEmailView(UserUpdateView):
    form_class = UserEmailForm

    def get_title(self):
        if self.request.user.email:
            return _('Change e-mail address')
        else:
            return _('Set e-mail address')


class UserPasswordView(auth_views.PasswordChangeView):
    template_name = 'leprikon/password.html'
    form_class = UserPasswordForm
    success_url = reverse_lazy('leprikon:summary')

    def get_context_data(self, **kwargs):
        kwargs.update({
            'submit_label': _('Change password'),
            'back_label': _('Back'),
            'back_url': reverse_lazy('leprikon:summary'),
            'placeholder': 'user_password',
        })
        return super().get_context_data(**kwargs)


class UserLoginView(auth_views.LoginView):
    template_name = 'leprikon/login.html'
    form_class = UserLoginForm
    redirect_field_name = settings.LEPRIKON_PARAM_BACK


class UserLogoutView(auth_views.LogoutView):
    next_page = settings.LOGOUT_REDIRECT_URL or '/'
    redirect_field_name = settings.LEPRIKON_PARAM_BACK


class PasswordResetView(auth_views.PasswordResetView):
    template_name = 'leprikon/password_reset.html'
    form_class = PasswordResetForm
    email_template_name = 'leprikon/password_reset_email.html'
    from_email = settings.SERVER_EMAIL

    def get_context_data(self, **kwargs):
        kwargs.update({
            'instructions': '<p>{}<p>'.format(
                _('Enter your email address, and we\'ll email instructions for setting a new one.')
            ),
            'submit_label': _('Reset my password'),
            'placeholder': 'password_reset',
        })
        return super().get_context_data(**kwargs)

    success_url = reverse_lazy('leprikon:password_reset_done')


class PasswordResetDoneView(auth_views.PasswordResetDoneView):
    template_name = 'leprikon/password_reset_done.html'


class PasswordResetConfirmView(auth_views.PasswordResetConfirmView):
    template_name = 'leprikon/password_reset_confirm.html'
    form_class = SetPasswordForm

    def get_context_data(self, **kwargs):
        kwargs.update({
            'instructions': '<p>{}<p>'.format(
                _('Please enter your new password twice so we can verify you typed it in correctly.')
            ),
            'submit_label': _('Set my password'),
            'placeholder': 'password_set',
        })
        return super().get_context_data(**kwargs)

    success_url = reverse_lazy('leprikon:password_reset_complete')


class PasswordResetCompleteView(auth_views.PasswordResetCompleteView):
    template_name = 'leprikon/password_reset_complete.html'



from functools import partial

from django import forms
from django.contrib import admin, messages
from django.forms.models import modelform_factory
from django.forms.widgets import CheckboxSelectMultiple
from django.shortcuts import render
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _
from sentry_sdk import capture_exception


class BulkUpdateMixin:
    actions = ('bulk_update',)

    bulk_update_fields = None
    bulk_update_exclude = None
    bulk_update_form = None

    @cached_property
    def BulkUpdateFieldsForm(self):
        return type('BulkUpdateFieldsForm', (forms.Form,), {
            'fields': forms.MultipleChoiceField(
                label=_('Fields to update'),
                choices=[
                    (field_name, field.label)
                    for field_name, field in forms.models.fields_for_model(
                        self.model,
                        fields=self.bulk_update_fields,
                        exclude=self.bulk_update_exclude,
                    ).items()
                ],
                widget=CheckboxSelectMultiple,
            ),
        })

    def bulk_update(self, request, queryset):
        phase = request.POST.get('phase')
        hidden_fields = []
        # phases: None > 'fields' > 'update'
        if phase is not None:
            form = self.BulkUpdateFieldsForm(request.POST)
            if form.is_valid():
                hidden_fields = [
                    ('fields', field)
                    for field in form.cleaned_data['fields']
                ]
                BulkUpdateForm = modelform_factory(
                    self.model,
                    form=self.bulk_update_form or self.form,
                    fields=form.cleaned_data['fields'],
                    formfield_callback=partial(self.formfield_for_dbfield, request=request),
                )
                if phase == 'update':
                    form = BulkUpdateForm(request.POST)
                    if form.is_valid():
                        num_updated = 0
                        for instance in queryset.all():
                            try:
                                instance_form = BulkUpdateForm(request.POST, request.FILES, instance=instance)
                                assert instance_form.is_valid()
                                instance_form.save()
                            except Exception:
                                capture_exception()
                                self.message_user(
                                    request,
                                    _(f'Failed to update item {instance}.'),
                                    messages.ERROR,
                                )
                            else:
                                num_updated += 1
                        self.message_user(request, _(f'{num_updated} items were updated.'))
                        return
                else:
                    form = BulkUpdateForm()
                    phase = 'update'
        else:
            form = self.BulkUpdateFieldsForm()
            phase = 'fields'
        hidden_fields.append(('phase', phase))
        return render(request, 'leprikon/admin/bulk_update_form.html', {
            'title': _('Bulk update'),
            'queryset': queryset,
            'opts': self.model._meta,
            'form': form,
            'media': self.media + form.media,
            'action': 'bulk_update',
            'action_checkbox_name': admin.helpers.ACTION_CHECKBOX_NAME,
            'hidden_fields': hidden_fields,
        })
    bulk_update.short_description = _('Bulk update selected items')



from django import forms
from django.contrib import admin
from django.contrib.admin.widgets import (
    AdminRadioSelect, FilteredSelectMultiple,
)
from django.contrib.auth import get_user_model
from django.contrib.staticfiles.templatetags.staticfiles import static
from django.db.models import F
from django.shortcuts import render
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from ..forms.courses import (
    CourseDiscountAdminForm, CourseRegistrationAdminForm,
)
from ..models.courses import (
    Course, CourseDiscount, CourseRegistration, CourseRegistrationHistory,
    CourseTime,
)
from ..models.schoolyear import SchoolYear, SchoolYearDivision
from ..models.subjects import SubjectType
from ..utils import currency
from .pdf import PdfExportAdminMixin
from .subjects import (
    SubjectBaseAdmin, SubjectPaymentBaseAdmin, SubjectRegistrationBaseAdmin,
)


class CourseTimeInlineAdmin(admin.TabularInline):
    model = CourseTime
    extra = 0


@admin.register(Course)
class CourseAdmin(SubjectBaseAdmin):
    subject_type_type = SubjectType.COURSE
    registration_model = CourseRegistration
    list_display = (
        'id', 'code', 'name', 'subject_type', 'get_groups_list', 'get_leaders_list',
        'get_times_list',
        'place', 'public', 'registration_allowed_icon',
        'get_registrations_link',
        'get_journal_link', 'icon', 'note',
    )
    list_export = (
        'id', 'school_year', 'code', 'name', 'department', 'subject_type', 'registration_type',
        'get_groups_list', 'get_leaders_list', 'get_age_groups_list', 'get_target_groups_list',
        'get_times_list', 'place', 'public', 'price',
        'min_participants_count', 'max_participants_count', 'min_group_members_count', 'max_group_members_count',
        'min_registrations_count', 'max_registrations_count',
        'get_approved_registrations_count', 'get_unapproved_registrations_count', 'note',
    )
    inlines = (
        CourseTimeInlineAdmin,
    ) + SubjectBaseAdmin.inlines
    actions = (
        'publish', 'unpublish',
        'copy_to_school_year',
    )

    def get_form(self, request, obj=None, **kwargs):
        form = super().get_form(request, obj, **kwargs)
        if 'school_year_division' in form.base_fields:
            # school year division choices
            school_year_division_choices = form.base_fields['school_year_division'].widget.widget.choices
            school_year_division_choices.queryset = SchoolYearDivision.objects.filter(
                school_year=obj.school_year if obj else request.school_year,
            )
            form.base_fields['school_year_division'].choices = school_year_division_choices
        return form

    def publish(self, request, queryset):
        Course.objects.filter(id__in=[reg['id'] for reg in queryset.values('id')]).update(public=True)
        self.message_user(request, _('Selected courses were published.'))
    publish.short_description = _('Publish selected courses')

    def unpublish(self, request, queryset):
        Course.objects.filter(id__in=[reg['id'] for reg in queryset.values('id')]).update(public=False)
        self.message_user(request, _('Selected courses were unpublished.'))
    unpublish.short_description = _('Unpublish selected courses')

    def copy_to_school_year(self, request, queryset):
        class SchoolYearForm(forms.Form):
            school_year = forms.ModelChoiceField(
                label=_('Target school year'),
                help_text=_('All selected courses will be copied to selected school year.'),
                queryset=SchoolYear.objects.all(),
            )
        if request.POST.get('post', 'no') == 'yes':
            form = SchoolYearForm(request.POST)
            if form.is_valid():
                school_year = form.cleaned_data['school_year']
                for course in queryset.all():
                    course.copy_to_school_year(school_year)
                self.message_user(request, _('Selected courses were copied to school year {}.').format(school_year))
                return
        else:
            form = SchoolYearForm()

        adminform = admin.helpers.AdminForm(
            form,
            [(None, {'fields': list(form.base_fields)})],
            {},
            None,
            model_admin=self,
        )

        return render(request, 'leprikon/admin/action_form.html', dict(
            title=_('Select target school year'),
            opts=self.model._meta,
            adminform=adminform,
            media=self.media + adminform.media,
            action='copy_to_school_year',
            action_checkbox_name=admin.helpers.ACTION_CHECKBOX_NAME,
            select_across=request.POST['select_across'],
            selected=request.POST.getlist(admin.helpers.ACTION_CHECKBOX_NAME),
        ))
    copy_to_school_year.short_description = _('Copy selected courses to another school year')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_registrations__subject__in=queryset,
        ).distinct()


class CourseRegistrationHistoryInlineAdmin(admin.TabularInline):
    model = CourseRegistrationHistory
    extra = 0
    readonly_fields = ('course',)

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


@admin.register(CourseRegistration)
class CourseRegistrationAdmin(PdfExportAdminMixin, SubjectRegistrationBaseAdmin):
    form = CourseRegistrationAdminForm
    subject_type_type = SubjectType.COURSE
    actions = ('add_discounts',)
    inlines = SubjectRegistrationBaseAdmin.inlines + (CourseRegistrationHistoryInlineAdmin,)
    list_display = (
        'variable_symbol', 'download_tag', 'subject_name', 'participants_list_html', 'price',
        'course_discounts', 'course_payments',
        'created_with_by', 'approved_with_by', 'payment_requested_with_by',
        'cancelation_requested_with_by', 'canceled_with_by',
        'note', 'random_number',
    )

    def get_queryset(self, request):
        return super().get_queryset(request).prefetch_related(
            'course_registration_periods',
        )

    def add_discounts(self, request, queryset):
        ABSOLUTE = 'A'
        RELATIVE = 'R'

        class DiscountBaseForm(forms.Form):
            discount_type = forms.ChoiceField(
                label=_('Discount type'),
                choices=(
                    (ABSOLUTE, _('absolute amount')),
                    (RELATIVE, _('relative amount in percents'))
                ),
                widget=AdminRadioSelect(),
            )
            amount = forms.DecimalField(label=_('Amount or number of percents'))
            explanation = CourseDiscount._meta.get_field('explanation').formfield()

        school_year_divisions = {
            school_year_division.id: school_year_division
            for school_year_division in SchoolYearDivision.objects.filter(
                courses__registrations__in=queryset,
            ).distinct()
        }

        label = _('Periods of school year division: {school_year_division}')
        DiscountForm = type('DiscountForm', (DiscountBaseForm,), {
            f'periods_{school_year_division.id}': forms.ModelMultipleChoiceField(
                label=label.format(
                    school_year_division=school_year_division.name,
                ),
                queryset=school_year_division.periods.all(),
                widget=FilteredSelectMultiple(
                    verbose_name=label.format(
                        school_year_division=school_year_division.name,
                    ),
                    is_stacked=False,
                ),
            )
            for school_year_division in school_year_divisions.values()
        })
        if request.POST.get('post', 'no') == 'yes':
            form = DiscountForm(request.POST)
            if form.is_valid():
                CourseDiscount.objects.bulk_create(
                    CourseDiscount(
                        registration=registration,
                        registration_period=registration_period,
                        amount=(
                            form.cleaned_data['amount']
                            if form.cleaned_data['discount_type'] == ABSOLUTE
                            else (
                                registration.price - sum(
                                    discount.amount
                                    for discount in registration.all_discounts
                                    if discount.registration_period_id == registration_period.id
                                )
                            ) * form.cleaned_data['amount'] / 100
                        ),
                        explanation=form.cleaned_data['explanation'],
                    )
                    for registration in queryset.annotate(
                        school_year_division_id=F('subject__course__school_year_division_id')
                    )
                    for registration_period in registration.course_registration_periods.filter(
                        period__in=form.cleaned_data[f'periods_{registration.school_year_division_id}']
                    )
                )
                self.message_user(request, _(
                    'The discounts have been created for selected registrations.'
                ))
                return
        else:
            form = DiscountForm()

        adminform = admin.helpers.AdminForm(
            form,
            [(None, {'fields': list(form.base_fields)})],
            {},
            None,
            model_admin=self,
        )

        return render(request, 'leprikon/admin/action_form.html', dict(
            title=_('Add discounts'),
            opts=self.model._meta,
            adminform=adminform,
            media=self.media + adminform.media,
            action='add_discounts',
            action_checkbox_name=admin.helpers.ACTION_CHECKBOX_NAME,
            select_across=request.POST['select_across'],
            selected=request.POST.getlist(admin.helpers.ACTION_CHECKBOX_NAME),
        ))
    add_discounts.short_description = _('Add discounts to selected registrations')

    def course_discounts(self, obj):
        html = []
        for status in obj.period_payment_statuses:
            query = f'?registration={obj.id}&registration_period={status.registration_period.id}'
            html.append(format_html(
                '{period}: <a href="{href}"><b>{amount}</b></a>',
                period=status.registration_period.period.name,
                href=reverse('admin:leprikon_coursediscount_changelist') + query,
                amount=currency(status.status.discount),
            ) + format_html(
                '<a class="popup-link" href="{href}" style="background-position: 0 0" title="{title}">'
                '<img src="{icon}" alt="+"/></a>',
                href=reverse('admin:leprikon_coursediscount_add') + query,
                title=_('add discount'),
                icon=static('admin/img/icon-addlink.svg'),
            ))
        return '<br/>'.join(html)
    course_discounts.allow_tags = True
    course_discounts.short_description = _('course discounts')

    def course_payments(self, obj):
        html = []
        for period in obj.period_payment_statuses:
            html.append(format_html(
                '{period}: <a class="popup-link" style="color: {color}" href="{href}" title="{title}">'
                '<b>{amount}</b></a>',
                period=period.registration_period.period.name,
                color=period.status.color,
                href=reverse('admin:leprikon_subjectpayment_changelist') + '?registration={}'.format(obj.id),
                title=period.status.title,
                amount=currency(period.status.paid),
            ) + format_html(
                '<a class="popup-link" href="{href}" style="background-position: 0 0" title="{title}">'
                '<img src="{icon}" alt="+"/></a>',
                href=reverse('admin:leprikon_subjectpayment_add') + '?registration={}'.format(obj.id),
                title=_('add payment'),
                icon=static('admin/img/icon-addlink.svg'),
            ))
        return '<br/>'.join(html)
    course_payments.allow_tags = True
    course_payments.short_description = _('course payments')


@admin.register(CourseDiscount)
class CourseDiscountAdmin(PdfExportAdminMixin, SubjectPaymentBaseAdmin):
    form = CourseDiscountAdminForm
    list_display = ('accounted', 'registration', 'subject', 'period', 'amount_html', 'explanation')
    list_export = ('accounted', 'registration', 'subject', 'period', 'amount', 'explanation')
    closed_fields = ('accounted', 'registration', 'period', 'amount')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-10-10 19:41
from __future__ import unicode_literals

from collections import defaultdict
from datetime import date
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


def create_courseregistration_periods(apps, schema_editor):
    CourseDiscount = apps.get_model('leprikon', 'CourseDiscount')
    CourseRegistration = apps.get_model('leprikon', 'CourseRegistration')
    CourseRegistrationPeriod = apps.get_model('leprikon', 'CourseRegistrationPeriod')
    SchoolYearPeriod = apps.get_model('leprikon', 'SchoolYearPeriod')

    today = date.today()
    periods = defaultdict(list)
    for period in SchoolYearPeriod.objects.all():
        period.first = period.school_year_division_id not in periods
        periods[period.school_year_division_id].append(period)

    for registration in CourseRegistration.objects.annotate(
        school_year_division_id=models.F('subject__course__school_year_division_id')
    ).iterator():
        CourseRegistrationPeriod.objects.bulk_create(
            CourseRegistrationPeriod(
                registration=registration,
                period=period,
                payment_requested=(
                    period.due_from < today
                    if registration.payment_requested
                    else False
                ),
            )
            for period in periods[registration.school_year_division_id]
            if period.end >= registration.created.date() and (
                registration.canceled is None or
                period.start <= registration.canceled.date() or
                period.first
            )
        )
    CourseDiscount.objects.update(
        registration_period_id=models.Subquery(
            CourseRegistrationPeriod.objects.filter(
                registration_id=models.OuterRef('registration_id'),
                period_id=models.OuterRef('period_id'),
            ).values('id')[:1]
        ),
    )
    invalid_course_discounts = CourseDiscount.objects.filter(registration_period=None)
    invalid_course_discounts.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0055_schoolyearblockplugin'),
    ]

    operations = [
        migrations.CreateModel(
            name='CourseRegistrationPeriod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('payment_requested', models.BooleanField(default=False, verbose_name='payment requested')),
                ('period', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='course_registration_periods', to='leprikon.SchoolYearPeriod', verbose_name='period')),
                ('registration', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='course_registration_periods', to='leprikon.CourseRegistration', verbose_name='registration')),
            ],
            options={
                'ordering': ('period__start',),
                'verbose_name': 'registered period',
                'verbose_name_plural': 'registered periods',
            },
        ),
        migrations.AddField(
            model_name='course',
            name='allow_period_selection',
            field=models.BooleanField(default=False, help_text='allow user to choose school year periods on registration form', verbose_name='allow period selection'),
        ),
        migrations.AddField(
            model_name='subject',
            name='min_due_date_days',
            field=models.PositiveIntegerField(default=3, verbose_name='minimal number of days to due date'),
        ),
        migrations.AddField(
            model_name='coursediscount',
            name='registration_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='discounts', to='leprikon.CourseRegistrationPeriod', verbose_name='period'),
        ),
        migrations.RunPython(create_courseregistration_periods),
    ]



from django.utils.translation import ugettext_lazy as _

'''
    This module contains strings from other packages, which are not correctly
    translated to Czech language.
'''

# cms
_('Publish page changes')



from django.forms.widgets import CheckboxSelectMultiple, RadioSelect


class RadioSelectBootstrap(RadioSelect):
    template_name = 'leprikon/widgets/multiple_input.html'
    option_template_name = 'leprikon/widgets/input_option.html'


class CheckboxSelectMultipleBootstrap(CheckboxSelectMultiple):
    template_name = 'leprikon/widgets/multiple_input.html'
    option_template_name = 'leprikon/widgets/input_option.html'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.23 on 2019-09-01 17:33
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0029_payment_restrictions'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='leprikonsite',
            name='old_registration_agreement',
        ),
        migrations.RemoveField(
            model_name='subject',
            name='old_registration_agreement',
        ),
        migrations.RemoveField(
            model_name='subjecttype',
            name='old_registration_agreement',
        ),
    ]



from django.contrib import admin

from ..models.organizations import Organization


@admin.register(Organization)
class OrganizationAdmin(admin.ModelAdmin):
    list_display = ('name', 'address', 'phone', 'email', 'company_num', 'vat_number', 'bank_account')



from collections import namedtuple
from itertools import chain

from django.template.response import TemplateResponse
from django.urls import reverse_lazy as reverse
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ...forms.reports.debtors import DebtorsForm
from ...models.courses import CourseRegistration
from ...models.events import EventRegistration
from ...models.orderables import OrderableRegistration
from ...views.generic import FormView


class ReportDebtorsView(FormView):
    form_class = DebtorsForm
    template_name = 'leprikon/reports/debtors.html'
    title = _('Debtors list')
    submit_label = _('Show')
    back_url = reverse('leprikon:report_list')

    ReportItem = namedtuple('ReportItem', ('registration', 'status'))

    class Report(list):
        @cached_property
        def sum(self):
            return sum(item.status for item in self)

    def form_valid(self, form):
        context = form.cleaned_data
        context['form'] = form
        context['reports'] = {}
        context['sum'] = 0

        for reg in chain.from_iterable(
            qs.filter(
                subject__school_year=self.request.school_year,
                approved__date__lte=context['date'],
            ).prefetch_related(
                'discounts',
                'payments',
            ).select_related('user')
            for qs in (
                CourseRegistration.objects.prefetch_related('course_registration_periods'),
                EventRegistration.objects,
                OrderableRegistration.objects,
            )
        ):
            status = reg.get_payment_status(context['date'])
            if status.amount_due:
                report = context['reports'].setdefault(reg.user, self.Report())
                report.append(self.ReportItem(registration=reg, status=status))
        context['sum'] = sum(
            report.sum
            for report in context['reports'].values()
        )

        return TemplateResponse(self.request, self.template_name, self.get_context_data(**context))



from cms.plugin_pool import plugin_pool
from django.utils.translation import ugettext as _

from ..models.orderables import (
    FilteredOrderableListPlugin, OrderableListPlugin, OrderablePlugin,
)
from .base import LeprikonPluginBase


@plugin_pool.register_plugin
class LeprikonOrderablePlugin(LeprikonPluginBase):
    name = _('Orderable event')
    model = OrderablePlugin
    raw_id_fields = ('orderable',)

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/orderable/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonOrderableListPlugin(LeprikonPluginBase):
    name = _('Orderable list')
    model = OrderableListPlugin
    filter_horizontal = ('age_groups', 'target_groups', 'groups', 'leaders')

    def get_render_template(self, context, instance, placeholder):
        return 'leprikon/cms/orderable_list/%s.html' % instance.template


@plugin_pool.register_plugin
class LeprikonFilteredOrderableListPlugin(LeprikonPluginBase):
    name = _('Orderable list with search form')
    model = FilteredOrderableListPlugin
    render_template = 'leprikon/cms/orderable_list_filtered.html'



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2019-12-01 19:04
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields
import localflavor.generic.models


def migrate_organization(apps, schema_editor):
    Organization = apps.get_model('leprikon', 'Organization')
    LeprikonSite = apps.get_model('leprikon', 'LeprikonSite')

    for leprikon_site in LeprikonSite.objects.all():
        leprikon_site.organization = Organization.objects.create(
            name=leprikon_site.company_name or leprikon_site.name,
            street=leprikon_site.street,
            city=leprikon_site.city,
            postal_code=leprikon_site.postal_code,
            email=leprikon_site.email,
            phone=leprikon_site.phone,
            company_num=leprikon_site.company_num,
            vat_number=leprikon_site.vat_number,
            iban=leprikon_site.iban,
            bic=leprikon_site.bic,
        )
        leprikon_site.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0037_message_sender'),
    ]

    operations = [
        migrations.CreateModel(
            name='Organization',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='organization name')),
                ('street', models.CharField(blank=True, max_length=150, null=True, verbose_name='street')),
                ('city', models.CharField(blank=True, max_length=150, null=True, verbose_name='city')),
                ('postal_code', leprikon.models.fields.PostalCodeField(blank=True, null=True, verbose_name='postal code')),
                ('email', leprikon.models.fields.EmailField(blank=True, max_length=254, null=True, verbose_name='email address')),
                ('phone', models.CharField(blank=True, max_length=30, null=True, verbose_name='phone')),
                ('company_num', models.CharField(blank=True, max_length=8, null=True, verbose_name='company number')),
                ('vat_number', models.CharField(blank=True, max_length=10, null=True, verbose_name='VAT number')),
                ('iban', localflavor.generic.models.IBANField(blank=True, include_countries=None, max_length=34, null=True, use_nordea_extensions=False, verbose_name='IBAN')),
                ('bic', localflavor.generic.models.BICField(blank=True, max_length=11, null=True, verbose_name='BIC (SWIFT)')),
            ],
            options={
                'verbose_name': 'organization',
                'verbose_name_plural': 'organizations',
            },
        ),
        migrations.AddField(
            model_name='leprikonsite',
            name='organization',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.Organization', verbose_name='default organization'),
        ),
        migrations.AddField(
            model_name='subject',
            name='organization',
            field=models.ForeignKey(blank=True, help_text='Only use to set value specific for this subject.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.Organization', verbose_name='organization'),
        ),
        migrations.AddField(
            model_name='subjecttype',
            name='organization',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.Organization', verbose_name='organization'),
        ),
        migrations.RunPython(migrate_organization),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='bic',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='city',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='company_name',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='company_num',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='email',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='iban',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='phone',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='postal_code',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='street',
        ),
        migrations.RemoveField(
            model_name='leprikonsite',
            name='vat_number',
        ),
        migrations.AlterField(
            model_name='leprikonsite',
            name='bill_print_setup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='default bill print setup'),
        ),
        migrations.AlterField(
            model_name='leprikonsite',
            name='reg_print_setup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='default registration print setup'),
        ),
        migrations.AlterField(
            model_name='leprikonsite',
            name='registration_agreements',
            field=models.ManyToManyField(blank=True, help_text='Add legal agreements for the registration form.', related_name='_leprikonsite_registration_agreements_+', to='leprikon.Agreement', verbose_name='default registration agreements'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='bill_print_setup',
            field=models.ForeignKey(blank=True, help_text='Only use to set value specific for this subject.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='payment print setup'),
        ),
        migrations.AlterField(
            model_name='subject',
            name='reg_print_setup',
            field=models.ForeignKey(blank=True, help_text='Only use to set value specific for this subject.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='leprikon.PrintSetup', verbose_name='registration print setup'),
        ),
    ]



from django import forms
from django.utils.translation import ugettext_lazy as _

from ...models.events import Event
from ..form import FormMixin


class EventPaymentsForm(FormMixin, forms.Form):
    date_start = forms.DateField(label=_('Start date'))
    date_end = forms.DateField(label=_('End date'))


class EventPaymentsStatusForm(FormMixin, forms.Form):
    date = forms.DateField(label=_('Status for date'))


class EventStatsForm(FormMixin, forms.Form):
    date = forms.DateField(label=_('Statistics for date'))
    events = forms.ModelMultipleChoiceField(
        queryset=Event.objects.all(),
        label=_('Events'),
    )
    paid_only = forms.BooleanField(label=_('Count only paid registrations'), required=False)

    def __init__(self, *args, **kwargs):
        school_year = kwargs.pop('school_year')
        super().__init__(*args, **kwargs)
        self.fields['events'].queryset = Event.objects.filter(school_year=school_year)



from cms.cms_toolbars import ADMIN_MENU_IDENTIFIER
from cms.toolbar.items import SubMenu
from cms.toolbar_base import CMSToolbar
from cms.toolbar_pool import toolbar_pool
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _


@toolbar_pool.register
class LeprikonToolbar(CMSToolbar):

    def populate(self):
        # Articles item in main menu
        admin_menu = self.toolbar.get_or_create_menu(ADMIN_MENU_IDENTIFIER)
        position = admin_menu.get_alphabetical_insert_position(_('Leprikon'), SubMenu)
        url = reverse('admin:app_list', args=('leprikon',))
        admin_menu.add_sideframe_item(_('Leprikon'), url=url, position=position)
